//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdio.h>
#include "ArmyDialog.h"
#include "defs.h"

ArmyDialog::ArmyDialog(Army* army, PG_Widget* parent, PG_Rect rect)
    :PG_Window(parent, rect, "Army info", WF_MODAL), d_army(army)
{
    char buffer[31];
    buffer[30] = '\0';
    unsigned int bonus;

    //I assume a size of 500*400 pixels here
    
    d_b_ok = new PG_Button(this, 1, PG_Rect(350, 360, 130, 30),
                            "OK");
    d_b_ok->SetEventObject(MSG_BUTTONCLICK, this,
                        (MSG_CALLBACK_OBJ)&ArmyDialog::b_ok_clicked);

    //the army pic
    d_b_pic = new PG_Button(this, 2, PG_Rect(100, 100, 50, 50), "");
    d_b_pic->EnableReceiver(false);
    d_b_pic->SetIcon(d_army->getPixmap(), 0, 0);

    //the name of the army
    d_l_name = new PG_Label(this, PG_Rect(250, 50, 200, 20), 
                            d_army->getName().c_str());
                            
    //the head of the table for the single values
    d_l_basic = new PG_Label(this, PG_Rect(400, 70, 45, 20), "basic");
    d_l_real = new PG_Label(this, PG_Rect(450, 70, 45, 20), "real");

    //and the values themselves
    d_l_strengthn = new PG_Label(this, PG_Rect(250, 90, 95, 20), "strength");
    snprintf(buffer, 30, "%i", d_army->getMaxStrength());
    d_l_strengthb = new PG_Label(this, PG_Rect(400, 90, 45, 20), buffer);
    snprintf(buffer, 30, "%i", d_army->getStrength());
    d_l_strengthr = new PG_Label(this, PG_Rect(450, 90, 45, 20), buffer);
    if (d_army->getMaxStrength() < d_army->getStrength())
        d_l_strengthr->SetFontColor(50, 255, 50);   //green highlight
    if (d_army->getMaxStrength() > d_army->getStrength())
        d_l_strengthr->SetFontColor(200, 30, 30);   //red highlight

    d_l_defensen = new PG_Label(this, PG_Rect(250, 110, 95, 20), "defense");
    snprintf(buffer, 30, "%i", d_army->getDefense());
    d_l_defenseb = new PG_Label(this, PG_Rect(400, 110, 45, 20), buffer);
    d_l_defenser = new PG_Label(this, PG_Rect(450, 110, 45, 20), buffer);

    d_l_vitalityn = new PG_Label(this, PG_Rect(250, 130, 96, 20), "vitality");
    snprintf(buffer, 30, "%i", d_army->getVitality());
    d_l_vitalityb = new PG_Label(this, PG_Rect(400, 130, 45, 20), buffer);
    d_l_vitalityr = new PG_Label(this, PG_Rect(450, 130, 45, 20), buffer);

    d_l_hpn = new PG_Label(this, PG_Rect(250, 150, 95, 20), "hp");
    snprintf(buffer, 30, "%i", d_army->getMaxHp());
    d_l_hpb = new PG_Label(this, PG_Rect(400, 150, 45, 20), buffer);
    snprintf(buffer, 30, "%i", d_army->getHP());
    d_l_hpr = new PG_Label(this, PG_Rect(450, 150, 45, 20), buffer);
    if (d_army->getHP() < d_army->getMaxHp())
        d_l_hpr->SetFontColor(200, 30, 30);

    d_l_movesn = new PG_Label(this, PG_Rect(250, 170, 95, 20), "moves");
    snprintf(buffer, 30, "%i", d_army->getMaxMoves());
    d_l_movesb = new PG_Label(this, PG_Rect(400, 170, 45, 20), buffer);
    snprintf(buffer, 30, "%i", d_army->getMoves());
    d_l_movesr = new PG_Label(this, PG_Rect(450, 170, 45, 20), buffer);
    if (d_army->getMoves() < d_army->getMaxMoves())
        d_l_movesr->SetFontColor(200, 30, 20);

    snprintf(buffer, 30, "xp: %i", d_army->getXP());
    d_l_xp = new PG_Label(this, PG_Rect(300, 200, 95, 20), buffer);
    snprintf(buffer, 30, "level: %i", d_army->getLevel());
    d_l_level = new PG_Label(this, PG_Rect(400, 200, 95, 20), buffer);

    snprintf(buffer, 30, "prod. time: %i turns", d_army->getProduction());
    d_l_prod = new PG_Label(this, PG_Rect(350, 270, 145, 20), buffer);
    snprintf(buffer, 30, "prod. cost: %i gold", d_army->getProductionCost());
    d_l_prodcost = new PG_Label(this, PG_Rect(350, 290, 145, 20), buffer);
    snprintf(buffer, 30, "Upkeep: %i gold",d_army->getUpkeep());
    d_l_upkeep = new PG_Label(this, PG_Rect(350, 310, 145, 20), buffer);

    //the move and army boni are a bit more complicated
    for (int i = 0; i < 4; i++)
        d_l_movebonusv[i] = 0;
        
    d_l_movebonusn = new PG_Label(this, PG_Rect(200, 230, 100, 20), "move bonus");
    bonus = d_army->getMoveBonus();
    if (bonus & MAPTILE_WATER)
    {
        d_l_movebonusv[0] = new PG_Label(this, PG_Rect(200, 260, 100, 20),
                                        "water");
        d_l_movebonusv[0]->SetFontColor(20, 20, 255);
    }
    if (bonus & MAPTILE_FOREST)
    {
        d_l_movebonusv[1] = new PG_Label(this, PG_Rect(200, 290, 100, 20),
                                        "forest");
        d_l_movebonusv[1]->SetFontColor(20, 255, 20);
    }
    if (bonus & MAPTILE_HILL)
    {
        d_l_movebonusv[2] = new PG_Label(this, PG_Rect(200, 320, 100, 20),
                                        "hill");
        d_l_movebonusv[2]->SetFontColor(150, 150, 150);
    }
    if (bonus & MAPTILE_SWAMP)
    {
        d_l_movebonusv[3] = new PG_Label(this, PG_Rect(200, 350, 100, 20),
                                        "swamp");
        d_l_movebonusv[3]->SetFontColor(150, 150, 0);
    }

    //army boni, something of a quick hack...
    for (int k = 0; k < 7; k++)
        d_l_armybonusv[k] = 0;
    d_l_armybonusn = new PG_Label(this, PG_Rect(30, 230, 100, 20), "bonus");
    bonus = d_army->getArmyBonus();
    if (bonus & ARMY_SHIP)
        d_l_armybonusv[0] = new PG_Label(this,PG_Rect(30,260, 150, 20), "ship");
    if (bonus & ARMY_HERO)
        d_l_armybonusv[1] = new PG_Label(this,PG_Rect(30,280, 150, 20), "hero");
    if (bonus & ARMY_CAVALRY)
        d_l_armybonusv[2] = new PG_Label(this, PG_Rect(30, 300, 150, 20), "cavalry");
    if (bonus & ARMY_ANTICAVALRY)
        d_l_armybonusv[3] = new PG_Label(this, PG_Rect(30, 320, 150, 20), "anticavalry");
    if (bonus & ARMY_HEALFAST)
        d_l_armybonusv[4] = new PG_Label(this, PG_Rect(30, 340, 150, 20), "healfast - NIY");
    if (bonus & ARMY_EVADE)
        d_l_armybonusv[5] = new PG_Label(this, PG_Rect(30, 360, 150, 20), "evade - NIY");
    if (bonus & ARMY_CRITICAL)
        d_l_armybonusv[6] = new PG_Label(this, PG_Rect(30, 380, 150, 20), "instant kills");
}

ArmyDialog::~ArmyDialog()
{
    delete d_b_ok;
    delete d_b_pic;
    delete d_l_name;
    delete d_l_basic;
    delete d_l_real;
    delete d_l_strengthn;
    delete d_l_strengthb;
    delete d_l_strengthr;
    delete d_l_defensen;
    delete d_l_defenseb;
    delete d_l_defenser;
    delete d_l_vitalityn;
    delete d_l_vitalityb;
    delete d_l_vitalityr;
    delete d_l_hpn;
    delete d_l_hpb;
    delete d_l_hpr;
    delete d_l_movesn;
    delete d_l_movesb;
    delete d_l_movesr;
    delete d_l_prod;
    delete d_l_prodcost;
    delete d_l_upkeep;
    delete d_l_xp;
    delete d_l_level;
    delete d_l_movebonusn;
    delete d_l_armybonusn;
    for (int i = 0; i < 4; i++)
        if (d_l_movebonusv[i])
            delete d_l_movebonusv[i];
    for (int i = 0; i < 7; i++)
        if (d_l_armybonusv[i])
            delete d_l_armybonusv[i];
}

PARAGUI_CALLBACK(ArmyDialog::b_ok_clicked)
{
    SendMessage(this, MSG_MODALQUIT, 0, 0);
    return true;
}
