//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef CREATE_SCENARIO_H
#define CREATE_SCENARIO_H

#include <string>
#include <sigc++/sigc++.h>
#include <SDL.h>

#include "player.h"
#include "playerlist.h"
#include "stacklist.h"
#include "citylist.h"
#include "ruinlist.h"
#include "templelist.h"
#include "armysetlist.h"
#include "MapGenerator.h"
#include "GamePreferencesDialog.h"

/* The purpose of this class is to create a new scenario which can then be
 * "loaded" by the game scenario class. The advantage is that the scenario
 * creation can be concentrated in one specific class instead of being scattered
 * over different classes and all objects (city, player, stack...) only need
 * one "loading" constructor. The disavantage is that all significant changes
 * e.g. to the player class have to be reflected here, too.
 */

class CreateScenario : public SigC::Object
{
    public:
        enum MapType {NORMAL};
            //this represents the class of the map (hills, islands etc.).
            //Currently, not much is going on here.
    
        CreateScenario(PG_Widget* uncle); //the widget is for the progress bar
        ~CreateScenario();

        //setters
        void setMaptype(MapType type);
        void setPercentages(int pgrass, int pwater, int pforest, int pswamp,
                            int phills, int pmountains);
            //set percentages of the single map types (diff to 100% is grass)
        void setMapTiles(std::string tilesname);
        void setNoCities(int number);
        void setNoRuins(int number);
        void setNoTemples(int number);
        void setSize(int size);
      
        void addPlayer(std::string name, std::string armyset, SDL_Color color,
                        Player::TYPE type);

        //getters
        int getNoPlayers();
        Player* getPlayer(int number);
        MapType getMaptype();
        int getNoCities();
        int getNoRuins();
        int getNoTemples();
        std::string getMapTiles();
        
        //now the real worker functions
        bool create();                      //creates a map...
        bool dump(std::string filename);    //...and dumps it

        //this signal will frequently be called whenever a step is completed
        SigC::Signal1<void, int> sstepfinished;

    private:
        //the single creation steps
        bool createMap();
        bool distributePlayers();
        bool setupCities();
        bool setupRuins();
        bool setupPlayers();

        //various helper functions
        bool loadCitynames();

        //data
        //for map creation
        MapGenerator* d_generator;
        std::string d_tilesname;
        int d_size;

        PG_Widget* d_uncle;

        //the namelists
        std::vector<std::string> d_citynames;
};

#endif  //CREATE_SCENARIO_H
