//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FILE_H
#define FILE_H

#include <string>
#include <list>
#include <SDL.h>
#include <pgdropdown.h>

// All file-related operations should use these functions.
// They are needed for installing FreeLords (no fixed directory structure)

class File
{
    public:
        // get all armysets we can offer
        static std::list<std::string> scanArmysets();

        // load army file/pic
        static std::string getArmyset(std::string armysetname);
        static SDL_Surface* getArmyPicture(std::string armysetname, bool alpha=true);
        //  if you want to load default_mask.png, issue default as armysetname
        static SDL_Surface* getArmyMask(std::string armysetname);

        // load mapset file/pic
        static std::string getMapset(std::string mapsetname);
        static SDL_Surface* getMapsetPicture(std::string mapsetname, std::string picname, bool alpha=true);

        // load misc file/pic
        static std::string getMiscFile(std::string filename);
        static SDL_Surface* getMiscPicture(std::string picname, bool alpha=true);

        //get save game path
        static std::string getSavePath();

        static std::list<std::string> s_armySetList;
        static std::list<std::string> s_tileSetList;

    	// methods search for tileSets 
        static std::list<std::string> scanTilesets(PG_DropDown* drop_down);

    private:
        static SDL_Surface* loadImage(std::string filename, bool alpha, bool colorkey=true);
};

#endif //FILE_H

// End of file
