//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MAPRENDERER_H
#define MAPRENDERER_H

#include <SDL.h>

class MapRenderer
{
    public:
        MapRenderer(SDL_Surface* surface);
        ~MapRenderer();

        // Draws the map starting at point(x,y) with Tile(tileX,tileY)
        // The area of tiles drawn are columns & rows.
        // This function can handle drawing areas outside the defined map by
        // simply using the default tile.
        void render(int x, int y, int tileX, int tileY, int columns, int rows);

    private:
        void smooth(int x, int y);
        int smoothCorner1(bool lt, bool rt, bool lb);
        int smoothCorner2(bool lt, bool rt, bool rb);
        int smoothCorner3(bool lt, bool lb, bool rb);
        int smoothCorner4(bool rt, bool lb, bool rb);
        void getCornerRect(SDL_Rect& rect, int i);

        SDL_Surface* d_surface;
};

#endif // MAPRENDERER_H

// End of file
