//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "QEnemyArmies.h"
#include "signals.h"
#include <sstream>

using namespace std;

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)
//=======================================================================
QuestEnemyArmies::QuestEnemyArmies(QuestsManager& q_mgr, int heroId)
                      : Quest(q_mgr, heroId),
                        d_how_many_killed(0)
{
    sdyingArmy.connect( SigC::slot(*this, &QuestEnemyArmies::dyingArmy));
    /** we have to kill 14-20 units: 14 + rand(0..6) */
    d_how_many_to_kill = 14 + (rand() % 7);
    debug("to kill: " << _how_many_to_kill);
    _initDescription(d_description, d_how_many_to_kill);
}
//=======================================================================
QuestEnemyArmies::QuestEnemyArmies(QuestsManager& q_mgr, int heroId,
                                   XML_Helper* helper) 
                                   : Quest(q_mgr, heroId)
{
    sdyingArmy.connect( SigC::slot(*this, &QuestEnemyArmies::dyingArmy));
    helper->getData(d_how_many_to_kill, "to_kill");
    helper->getData(d_how_many_killed,  "killed");
    _initDescription(d_description, d_how_many_to_kill);
}
//=======================================================================
bool QuestEnemyArmies::save(XML_Helper *helper)
{
    bool retval = true;
	retval &= helper->saveData("to_kill", d_how_many_to_kill);
	retval &= helper->saveData("killed",  d_how_many_killed);
    return retval;
}
//=======================================================================
void QuestEnemyArmies::_initDescription(std::string& d, 
                                        unsigned int how_many_to_kill)
{
	std::stringstream s;
	s << "You must destroy " << d_how_many_to_kill 
      << " units of the enemy armies";
    d_description = s.str();
}
//=======================================================================
bool QuestEnemyArmies::isFeasible()
{
    return true;
}
//=======================================================================
void QuestEnemyArmies::getProgress(std::string& dst)
{
	std::stringstream s;
    s << "You have killed " << d_how_many_killed << " so far";
    dst += s.str();
}
//=======================================================================
void QuestEnemyArmies::dyingArmy(Army *army, culpritsType culprits)
{
    debug("QuestEnemyArmies: dyingArmy - pending = " << (int)pending);
    if (pending == 0)
        return;
    // did our hero kill them?
    int ourHero = getHeroId();
    culpritsType::iterator it;
    for (it = culprits.begin(); it != culprits.end(); it++)
    {
        if ((*it) == ourHero)
        {
            d_how_many_killed++;
            if (d_how_many_killed == d_how_many_to_kill)
            {
                debug("CONGRATULATIONS: QUEST 'ENEMY ARMIES' IS COMPLETED!");
                questCompleted();
            }
            break;
        }
    }
}
//=======================================================================
void QuestEnemyArmies::getSuccessMsg(std::queue<std::string>& msgs)
{
	std::stringstream s;
    s << "You have managed to slaughter " << d_how_many_killed << " armies";
    msgs.push(s.str());
    msgs.push("Well done!");
}
