//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "Tile.h"
#include <iostream>

using namespace std;

Tile::Tile()
{
    SDL_Surface* screen = SDL_GetVideoSurface();
    SDL_PixelFormat* fmt = screen->format;
    for (int i = 0; i < 8*4; i++)
    {
        // remove the hardcoded 25, but who should know the tilesize?
        // the tile, or the tileset?
        SDL_Surface* tmp = SDL_CreateRGBSurface(SDL_HWSURFACE, 25, 25, 16,
                fmt->Rmask, fmt->Gmask, fmt->Bmask, fmt->Amask);
        if (!tmp)
        {
            cerr << "Couldn't create surface! Probably out of memory!" << endl;
            cerr << "Exiting!" << endl;
            exit(1);
        }
        d_surface[i] = SDL_DisplayFormat(tmp);
        SDL_FreeSurface(tmp);
    }
}
	
Tile::~Tile()
{
    for (int i = 0; i < 8*4; i++)
    {
        SDL_FreeSurface(d_surface[i]);
    }
}

SDL_Surface* Tile::getSurface(int t)
{
    return d_surface[t];
}

void Tile::printDebugInfo()
{
    cout << "name = " << d_name << endl;
    cout << "moves = " << d_moves << endl;
    cout << "defense = " << d_defense << endl;
}

// End of file
