//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef TILE_H
#define TILE_H

#include <string>
#include <SDL.h>

class Tile
{
	public:
		// CREATORS
		Tile();
		~Tile();

		// MANIPULATORS
		void setName(std::string name){d_name = name;}
		void setMoves(int moves){d_moves = moves;}
		void setDefense(int defense){d_defense = defense;}
		void setColor(SDL_Color& color){d_color = color;}
		void setSurface(int t, SDL_Surface* surface){d_surface[t] = surface;}
		void setType(unsigned int type) {d_type = type;}
        void printDebugInfo();

		// ACCESSORS
		SDL_Surface* getSurface(int t);
		std::string getName(){return d_name;}
		int getMoves(){return d_moves;}
		int getDefense(){return d_defense;}
		SDL_Color getColor(){return d_color;}
		unsigned int getType(){return d_type;}
				
	private:
		// DATA
		SDL_Surface* d_surface[8*4];  	// base tiles * 4 = corners
		std::string d_name;	    	    // name	
		int d_moves;		        	// moves needed to walk over maptile
		int d_defense;		    	    // defense bonus
		SDL_Color d_color;	    	    // color shown in the smallmap
		unsigned int d_type;
};

#endif // TILE_H

// End of file
