//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdlib.h>
#include <algorithm>
#include "ai_fast.h"
#include "playerlist.h"
#include "armysetlist.h"
#include "FightDialog.h"
#include "stacklist.h"
#include "citylist.h"
#include <fstream>
#include "path.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

AI_Fast::AI_Fast(string name, string armyset, SDL_Color color)
	:RealPlayer(name, armyset, color, Player::AI_FAST)
{
}

AI_Fast::AI_Fast(const Player& player)
	:RealPlayer(player)
{
}

AI_Fast::AI_Fast(XML_Helper* helper)
	:RealPlayer(helper)
{
}

AI_Fast::~AI_Fast()
{
}

bool AI_Fast::startTurn()
{
    debug("AI_Fast::start_turn")
   
    sstartingTurn.emit();
	clearActionlist();

	//this is a recursively-programmed quite staightforward AI,we just call:
	computerTurn();

    d_stacklist->setActivestack(0);

	nextTurn();

	return true;
}

bool AI_Fast::nextTurn()
{
    debug("AI_Fast::nextTurn");

    sendingTurn.emit(getType());
	Playerlist::getInstance()->nextPlayer();

	return true;
}

bool AI_Fast::invadeCity(City* c)
{
    //stupid ai player always occupies an enemy city
    bool retval = cityOccupy(c);
    sinvadingCity.emit(c);

    return retval;
}

void AI_Fast::computerTurn()
{
    /* This is how scripting should work (this code actualy really works ;))
    string command;
    while (command != "ENDTURN")
    {
        command = getAICommand();

        // do all different computer actions here
    
        if (command == "MOVE")
        {
            Stack* s = stacklist->set_next();
            PG_Point p;
            p.x = p.y = 10;
            s->get_path()->calculate(s, p);
            stacklist->move(s);
        }
    }
    */
    
    // easy AI player, but very fast!
	for (Stacklist::iterator it = d_stacklist->begin();
        it != d_stacklist->end(); ++it)
	{
	    d_stacklist->setActivestack(*it);
	    Stack* s = *it;
		if (s->size() >= 8)
        {
             PG_Point p = Citylist::getInstance()->getNearestEnemyCity(s->getPos())->getPos();
             s->getPath()->calculate(s, p);
             stackMove(s);
        }
        // a stack has died -> further iterating leads to a crash so we 
        // start again (recursive programming, therefore the break
        // afterwards is _VERY_ important)
        if (!d_stacklist->getActivestack())
        {
            computerTurn();
            break;
        }
	}
}

// End of file
