//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef ARMY_H
#define ARMY_H

#include <string>
#include "armyset_army.h"
#include "xmlhelper.h"

class Armyset;
class Player;

// This class contains all data and operations related to an Army. It's mostly
// used by the class Stack

class Army : public Armyset_army
{
    public:
        // CREATORS
        Army(Armyset* armyset, int type, Player* owner);
        Army(Army& a);
        Army(Armyset* armyset, int type, XML_Helper* helper);
        ~Army();

        // MANIPULATORS
        void setType(int type){d_type = type;}
        void setOwner(Player* owner) {d_owner = owner;}
        void setHP(int hp) {d_hp = hp;}
        void setGrouped(bool grouped){d_grouped = grouped;}

        // ACCESSORS
        int getId() const {return d_id;}
        int getType() const {return d_type;}
        Player* getOwner() const {return d_owner;}
        SDL_Surface* getPixmap();
        int getHP() const {return d_hp;}
        int getMoves() const {return d_moves;}
        int getStrength() const {return d_strength;}
        int getXP() const {return d_xp;}
        int getLevel() const {return d_level;}
        bool isBlessed() const {return d_blessed;}
        bool isGrouped() const {return d_grouped;}
		virtual bool isHero() {return false;}

        //MISC FUNCTIONS
        void heal(int hp = 0);
        bool damage(int damageDone);		//true, if army has died
        void decrementMoves(int moves);
        void resetMoves();
        void bless();
        void gainXp(int n);
        virtual bool save(XML_Helper* helper);
		
    protected:
        Armyset* d_armyset;
        Player* d_owner;
        unsigned int d_id;
        int d_type;
        int d_hp;
        int d_moves;
        int d_strength;
        int d_xp;			// experience points
        int d_level;
        bool d_blessed;			// if army is blessed then strength is +1
        bool d_grouped;
};

#endif // ARMY_H
