//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef ARMYSET_ARMY_H
#define ARMYSET_ARMY_H

#include <SDL.h>
#include <string>

// This class is the atom of every armyset. It contains all data related to
// a single armytype of an armyset

class Armyset_army
{
	public:
		// CREATORS
        Armyset_army();
		Armyset_army(const Armyset_army& a);
		~Armyset_army();

		// MANIPULATORS
		void setName(std::string name){d_name = name;}
		void setPixmap(SDL_Surface* pixmap){d_pixmap = pixmap;}
        void setMask(SDL_Surface* mask){d_mask = mask;}
		void setMaxHp(int max_hp){d_max_hp = max_hp;}
		void setMaxStrength(int max_strength){d_max_strength = max_strength;}
		void setDefense(int defense){d_defense = defense;}
		void setVitality(int vitality){d_vitality = vitality;}
		void setMaxMoves(int max_moves){d_max_moves = max_moves;}
		void setUpkeep(int upkeep){d_upkeep = upkeep;}
		void setProduction(int production){d_production = production;}
		void setProductionCost(int production_cost){d_production_cost = production_cost;}
		void setMoveBonus(unsigned int move_bonus){d_move_bonus = move_bonus;}
		void setArmyBonus(unsigned int army_bonus){d_army_bonus = army_bonus;}
		void setXpWorth (int xp_worth){d_xp_worth = xp_worth;}
		void printDebugInfo();
		
		// ACCESSORS
		std::string getName() const {return d_name;}
		virtual SDL_Surface* getPixmap() const {return d_pixmap;}
        SDL_Surface* getMask() const {return d_mask;}
		int getMaxStrength() const {return d_max_strength;}
		int getMaxMoves() const {return d_max_moves;}
		int getUpkeep() const {return d_upkeep;}
		int getProduction() const {return d_production;}
		int getDefense() const {return d_defense;}
		int getVitality() const {return d_vitality;}
		int getProductionCost() const {return d_production_cost;}
		int getMaxHp() const {return d_max_hp;}
		unsigned int getMoveBonus() const {return d_move_bonus;}
		unsigned int getArmyBonus() const {return d_army_bonus;}
		int getXpWorth() const {return d_xp_worth;}
		
	private:
		std::string d_name;
		SDL_Surface* d_pixmap;
        SDL_Surface* d_mask;
		int d_production;		// needed turns to produce this armytype
		int d_production_cost;
		int d_upkeep;
		int d_max_strength;
		int d_defense;
		int d_max_hp;
		int d_max_moves;
		int d_vitality;
		unsigned int d_move_bonus;
		unsigned int d_army_bonus;
		int d_xp_worth;
};

#endif // ARMYSET_ARMY_H

// End of file
