//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "hero_offer.h"
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <pgbutton.h>
#include <pglineedit.h>
#include <pglabel.h>
#include "playerlist.h"
#include "citylist.h"
#include "city.h"
#include "armyset.h"
#include "player.h"
#include "File.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

Hero_offer::Hero_offer(int gold, PG_Widget* parent)
	:PG_Window(parent, PG_Rect(0, 0, 210, 300), "Hero offer"),
	d_gold(gold), d_city(0)
{
    debug("Hero_offer::Hero_offer")

	MoveWindow((PG_Application::GetScreenWidth() - my_width) / 2,
			(PG_Application::GetScreenHeight() - my_height) / 2 - 50);

	// first city of activeplayer
	for (Citylist::iterator it = Citylist::getInstance()->begin();
			it != Citylist::getInstance()->end(); it++)
	{
		if ((*it).getPlayer() == Playerlist::getInstance()->getActiveplayer())
		{
			d_city = &(*it);
			break;
		}
	}

	d_gender = static_cast<GENDER> (rand() % 2);

    if (d_gender == MALE)
        d_recruitpic = File::getMiscPicture("recruit_male.png", false);
    else
        d_recruitpic = File::getMiscPicture("recruit_female.png", false);

	d_l_pic = new PG_Label(this, PG_Rect(5, 30, 200, 200), "");
	d_l_pic->SetIcon(d_recruitpic);

	d_b_accept = new PG_Button(this, 1, PG_Rect(10, my_height - 30, 70, 20), "Accept");
	d_b_accept->SetEventObject(MSG_BUTTONCLICK, this,
			(MSG_CALLBACK_OBJ)&Hero_offer::b_acceptClicked);

	d_b_reject = new PG_Button(this, 2, PG_Rect(my_width - 80, my_height - 30, 70, 20), "Reject");
	d_b_reject->SetEventObject(MSG_BUTTONCLICK, this,
			(MSG_CALLBACK_OBJ)&Hero_offer::b_rejectClicked);

    //take a random name from a hero name file. This is an ugly hack but
    //better than having to rely on a python script which produces a temporary
    //file which ...
    //This can and has to be improved (male vs. female names etc.), but works
    //for now
    //TODO: improve this
    string new_name = "Demonlord";
    fstream name_file(File::getMiscFile("heronames").c_str());
    if (name_file)
    {
        int entries;
        name_file >> entries;

        if (entries != 0)
            entries = (rand() % entries);
        for (int i = 0; i < entries; i++)
            name_file >> new_name;

        //now take the real name
        name_file >> new_name;
    }

	d_e_name = new PG_LineEdit(this, PG_Rect(40, 240, 120, 20));
	d_e_name->SetText(new_name.c_str());

	char text[80], text2[80];

    if (gold)
    {
        strcpy(text, "A hero wants to join you");
        sprintf(text2, "for %i gold", gold);
    }
    else
    {
        strcpy(text, "A hero wants to join for free!");
        strcpy(text2, "");
    }

	d_l_text = new PG_Label(this, PG_Rect(10, 195, 180, 20), text);
    d_l_text2 = new PG_Label(this, PG_Rect(10, 215, 180, 20), text2);
}

Hero_offer::~Hero_offer()
{
	delete d_b_accept;
	delete d_b_reject;
	delete d_e_name;
	delete d_l_pic;
	delete d_l_text;
    delete d_l_text2;
    SDL_FreeSurface(d_recruitpic);
}

PARAGUI_CALLBACK(Hero_offer::b_acceptClicked)
{
	debug("b_accept_clicked()");

	d_city->addHero(d_gender, d_e_name->GetText());
	Playerlist::getInstance()->getActiveplayer()->withdrawGold(d_gold);

	SendMessage(this, MSG_MODALQUIT, 0, 0);
	return true;
}

PARAGUI_CALLBACK(Hero_offer::b_rejectClicked)
{
	SendMessage(this, MSG_MODALQUIT, 0, 0);
	return true;
}

