//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "questsreport.h"
#include <stdio.h>
#include "playerlist.h"
#include "QuestsManager.h"
#include <sstream>

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

//first comes the implementation of StackItem. A StackItem is a bar of 420x80
//pixels which displays infos about a single stack.
class QuestItem : public SigC::Object, public PG_EventObject, 
                  public PG_Widget
{
    public:
        QuestItem(PG_Widget* parent, PG_Rect rect);
        ~QuestItem();

        void fillItem(Quest* quest, Hero *hero);

        SigC::Signal1<void, Quest*> sclicked;

    private:
        bool eventMouseButtonDown(const SDL_MouseButtonEvent * event);

        PG_Label* d_l_hero;
        PG_Label* d_l_quest;
        PG_Label* d_l_status;
        Quest* d_quest;
};

QuestItem::QuestItem(PG_Widget* parent, PG_Rect rect)
                    :PG_Widget(parent, rect)
{
    d_l_hero = new PG_Label(this, PG_Rect(10, 10, 150, 15), "");
    d_l_quest = new PG_Label(this, PG_Rect(10, 30, 480, 15), "");
    d_l_status = new PG_Label(this, PG_Rect(10, 50, 480, 15), "");
}

QuestItem::~QuestItem()
{
    delete d_l_hero;
    delete d_l_quest;
    delete d_l_status;
}

void QuestItem::fillItem(Quest* quest, Hero *hero)
{
    d_quest = quest;
    Player *player = Playerlist::getActiveplayer();

    SDL_Color color = player->getColor();
    d_l_hero->SetFontColor(color.r, color.g, color.b);

	std::stringstream s;
	s << hero->getName() << "(" << hero->getId() << ")";
    d_l_hero->SetText(s.str().c_str());

    d_l_quest->SetText(quest->getDescription());
    std::string progress;
    quest->getProgress(progress);
    d_l_status->SetText(progress.c_str());
}

bool QuestItem::eventMouseButtonDown(const SDL_MouseButtonEvent* event)
{
    if (!IsVisible())
        return true;
    sclicked.emit(d_quest);
    return true;
}


QuestsReport::QuestsReport(PG_Widget* parent, PG_Rect rect)
    :PG_Window(parent, rect, "Quests report", WF_MODAL), d_index(0)
{
    //I assume 500x400 pixels here as well
    d_b_ok = new PG_Button(this, 2, PG_Rect(450, 370, 45, 20), "OK");
    d_b_ok->SetEventObject(MSG_BUTTONCLICK, this,
                            (MSG_CALLBACK_OBJ)&QuestsReport::b_okClicked);

    d_b_up = new PG_Button(this, 0, PG_Rect(450, 150, 45, 20), "Prev.");
    d_b_up->SetEventObject(MSG_BUTTONCLICK, this,
                            (MSG_CALLBACK_OBJ)&QuestsReport::b_upClicked);

    d_b_down = new PG_Button(this, 1, PG_Rect(450, 250, 45, 20), "Next");
    d_b_down->SetEventObject(MSG_BUTTONCLICK, this,
                            (MSG_CALLBACK_OBJ)&QuestsReport::b_downClicked);

    d_l_number = new PG_Label(this, PG_Rect(460, 35, 45, 20), "");

    d_items[0] = new QuestItem(this, PG_Rect(10, 30, 420, 80));
    d_items[1] = new QuestItem(this, PG_Rect(10, 120, 420, 80));
    d_items[2] = new QuestItem(this, PG_Rect(10, 210, 420, 80));
    d_items[3] = new QuestItem(this, PG_Rect(10, 300, 420, 80));

    d_l_noquests = new PG_Label(this, PG_Rect(10, 35, 480, 40), "");

    Player *player = Playerlist::getActiveplayer();
    // read quests for this player
    QuestsManager *q_mgr = QuestsManager::getInstance();
    q_mgr->getPlayerQuests(player, d_questlist, d_heroes);
    d_index = 0;
    fillQuestItems();
}

QuestsReport::~QuestsReport()
{
    delete d_l_noquests;

    delete d_items[0];
    delete d_items[1];
    delete d_items[2];
    delete d_items[3];

    delete d_l_number;
    delete d_b_down;
    delete d_b_up;
    delete d_b_ok;
}

PARAGUI_CALLBACK(QuestsReport::b_okClicked)
{
    SendMessage(this, MSG_MODALQUIT, 0, 0);
    return true;
}

PARAGUI_CALLBACK(QuestsReport::b_upClicked)
{
    d_index--;
    if(d_index < 0)
        d_index = 0;
    return true;
}

PARAGUI_CALLBACK(QuestsReport::b_downClicked)
{
    d_index++;
    if ((d_index+4) > d_questlist.size())
        d_index = d_questlist.size() - 4;
    fillQuestItems();
    return true;
}

void QuestsReport::fillQuestItems()
{
    debug("fillQuestItems: count = " << d_heroes.size());
    if (d_heroes.size() > 0)
    {
        unsigned int max = d_index + 4;
        if (max > d_heroes.size())
            max = d_heroes.size();
        d_l_number->SetTextFormat("%i - %i", d_index+1, max);
    }
    else
    {
        Player *player = Playerlist::getActiveplayer();
        d_l_noquests->SetTextFormat("No quests for %s", 
                                    player->getName().c_str());
    }
    for (unsigned int i = 0; i < 4; i++)
    {
        unsigned int idx = d_index+i;
        if (idx >= d_heroes.size())
            return;
        debug("fillItem: i = " << i << ", idx " << idx);
        d_items[i]->fillItem(d_questlist[idx], d_heroes[idx]);
        d_items[i]->Redraw();
    }
}
