//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef STACK_H
#define STACK_H

#include <list>
#include "xmlhelper.h"
#include "paragui.h"
#include "signals.h"

class Player;
class Path;
class Army;

class Stack : public std::list<Army*>, public SigC::Object
{
	public:
		// CREATORS
		Stack(Player* player, PG_Point pos);
		Stack(Stack& s);
        Stack(XML_Helper* helper);
		~Stack();

		// MANIPULATORS
		void setDefending(bool defending){d_defending = defending;}
		void nextTurn();
		void decrementMoves(int moves);
		bool moveOneStep();
		void resetMoves();
		void upkeep();
		void bless();

		// ACCESSORS
		unsigned int getId() const {return d_id;}
		Player* getPlayer() const {return d_player;}
        PG_Point getPos() {return d_pos;}
		Path* getPath() const {return d_path;}
		int getGroupMoves();
		int getGroupMoves(unsigned int mapType);
		PG_Point nextStep();
		Army* getStrongestArmy() const;
		Army* getFirstUngroupedArmy() const;
		bool hasHero() const;
	    Army* getFirstHero() const;
        void getHeroes(culpritsType& dst);
		bool isEnemy(Player* player) const;
		bool isFriend(Player* player) const;
		bool getDefending() const {return d_defending;}

		//MISCELLANEOUS
		bool save(XML_Helper* helper);
        bool load(std::string tag, XML_Helper* helper);
        
		void flClear();
		iterator flErase(iterator object);
		
	private:	
		// DATA
		unsigned int d_id;
		Player* d_player;
		Path* d_path;
		bool d_defending;
        PG_Point d_pos;
};

#endif // STACK_H

// End of file
