//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef STACKLIST_H
#define STACKLIST_H

#include <list>
#include <pgmessageobject.h>
#include <pgeventobject.h>
#include <sigc++/object_slot.h>

class City;
class Stack;
class XML_Helper;

//The list which contains all army stacks. It belongs (not yet, but soon) to
//the player owning the stacks. We further need some static functions which
//search through all stakcs, i.e. through all stacklists of all players.

class Stacklist : public std::list<Stack*>, public PG_EventObject, public SigC::Object
{
	public:
        static Stack* getObjectAt(int x, int y);
        static Stack* getObjectAt(PG_Point point);
        //look if another stack occupies the same point as stack "s"
        //(need to force joining of stacks when moving)
        static Stack* getAmbiguity(Stack* s);

        //some other handy or elswhere neccessary static functions
        static void deleteStack(Stack* s);
        static void nextTurn();
		static std::vector<Stack*> defendersInCity(City* c);
        static unsigned int getNoOfStacks();

		// CREATORS
		Stacklist();
        Stacklist(XML_Helper* helper);
		~Stacklist();

		// MANIPULATORS
		void setActivestack(Stack* activestack) {d_activestack = activestack;}
		Stack* setNext();
		bool canJoin(Stack* s, Stack* d);
        
		bool save(XML_Helper* helper);
        bool load(std::string tag, XML_Helper* helper);
		
		// ACCESSORS
		Stack* getActivestack() const {return d_activestack;}

        //remove methods which also frees the memory of the pointers
		void flClear();
		iterator flErase(iterator object);
		bool flRemove(Stack* object);

	private:
		Stack* d_activestack;
};

#endif // STACKLIST_H

// End of file
