//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <assert.h>
#include <unistd.h>    //for usleep
#include <pgmessagebox.h>
#include "w_edit.h"
#include "RuinSearchDialog.h"
#include "CityOccupationDialog.h"
#include "FightDialog.h"
#include "bigmap.h"
#include "smallmap.h"
#include "army.h"
#include "hero.h"
#include "stacklist.h"
#include "citylist.h"
#include "ruinlist.h"
#include "templelist.h"
#include "ruin.h"
#include "temple.h"
#include "GameMap.h"
#include "playerlist.h"
#include "stackinfo.h"
#include "hero_offer.h"
#include "path.h"
#include "Configuration.h"
#include "File.h"
#include "NextPlayerPopup.h"
#include "player.h"
#include "signals.h"
#include "QuestsManager.h"

using namespace std;

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)

W_Edit::W_Edit(GameScenario* gameScenario, PG_Widget* parent, PG_Rect rect)
        :PG_Widget(parent, rect), d_gameScenario(gameScenario), d_lock(false)
{
    // this defines how many squares the bigmap shows! this is important
    // for the smallmap, because the viewrect depends on it
    int x_squares = (rect.my_width - 139) / 50;
    int y_squares = (rect.my_height - 110) / 50;

    // load button pictures
    d_pic_next = File::getMiscPicture("button_next.png", false);
    d_pic_defend = File::getMiscPicture("button_guard.png", false);
    d_pic_move = File::getMiscPicture("button_move.png", false);
    d_pic_defendAndNext = File::getMiscPicture("button_guardnext.png", false);
    d_pic_search = File::getMiscPicture("button_search.png", false);
    d_pic_nextTurn = File::getMiscPicture("button_endturn.png", false);
    d_pic_centerCurrent = File::getMiscPicture("button_center.png", false);

    //some graphical finetuning: place the smallmap in the middle of the piece
    //above the buttons (in the middle of x = (width - 109) -- (width -7) and
    //y = 20 -- 122
    PG_Rect smallrect;
    smallrect.x = rect.my_width - 58 - (GameMap::getSize()/2);
    smallrect.y = 71 - (GameMap::getSize()/2);
    smallrect.w = GameMap::getSize() + 2;
    smallrect.h = smallrect.w;
    d_smallmap = new SmallMap(this, smallrect, x_squares, y_squares);

    d_bigmap = new BigMap(this, 
            PG_Rect(20, 20, x_squares*50, y_squares*50));
    d_bigmap->setViewrect(d_smallmap->getViewrect());
    d_stackinfo = new Stackinfo(this, 
            PG_Rect(20, rect.my_height - 80, 360, 72));
    d_nextPlayerPopup = new NextPlayerPopup(this, 
            PG_Rect((my_width-300)/2, (my_height-200)/2, 300, 200));
    d_nextPlayerPopup->Hide();

    l_turns = new PG_Label(this,
            PG_Rect(rect.my_width - 109, 130, 100, 20), "");
    l_gold = new PG_Label(this,
            PG_Rect(rect.my_width - 109, 150, 100, 20), "");

    b_move = new PG_Button(this, 1,
            PG_Rect(rect.my_width - 109, 170, 40, 40));
    b_next = new PG_Button(this, 2,
            PG_Rect(rect.my_width - 69, 170, 40, 40));
    b_defend = new PG_Button(this, 3,
            PG_Rect(rect.my_width - 109, 210, 40, 40));
    b_defendAndNext = new PG_Button(this, 4,
            PG_Rect(rect.my_width - 69, 210, 40, 40));
    b_search = new PG_Button(this, 5,
            PG_Rect(rect.my_width - 109, 250, 40, 40));
    b_nextTurn = new PG_Button(this, 6,
            PG_Rect(rect.my_width - 69, 250, 40, 40));
    b_centerCurrent = new PG_Button(this, 7,
            PG_Rect(rect.my_width - 109, 290, 40, 40));


    sdyingStack.connect(SigC::slot((*this), &W_Edit::stackDied));
	// this has to be done for each player!!!
	for (Playerlist::iterator it = Playerlist::getInstance()->begin();
			it != Playerlist::getInstance()->end(); ++it)
	{
        (*it)->schangingStatus.connect(SigC::slot((*this), &W_Edit::updateStatus));
        (*it)->supdatingStack.connect(SigC::slot((*this), &W_Edit::stackUpdate));

        (*it)->sinvadingCity.connect(SigC::slot((*this), &W_Edit::cityOccupied));
        (*it)->sinterruptTimers.connect(SigC::slot((*this), &W_Edit::stopTimers));
        (*it)->scontinueTimers.connect(SigC::slot((*this), &W_Edit::startTimers));
        
        if ((*it)->getType() == Player::HUMAN)
        {
            (*it)->sstartingTurn.connect(SigC::slot((*this), 
                                                     &W_Edit::unlockScreen));
        }
	}
    sendingTurn.connect(SigC::slot((*this), &W_Edit::endingTurn));


    d_smallmap->schangingViewrect.connect(SigC::slot((*this), &W_Edit::bigmapRedraw));
    d_bigmap->schangingViewrect.connect(SigC::slot((*this), &W_Edit::smallmapRedraw));
    d_bigmap->sselectingStack.connect(SigC::slot((*this), &W_Edit::bigmapStackSelected));
    
    b_move->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_moveClicked);
    b_next->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_nextClicked);
    b_defend->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_defendClicked);
    b_defendAndNext->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_defendAndNextClicked);
    b_search->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_searchClicked);
    b_nextTurn->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_nextTurnClicked);
    b_centerCurrent->SetEventObject(MSG_BUTTONCLICK, this,
                     (MSG_CALLBACK_OBJ)&W_Edit::b_centerCurrentClicked);

    b_nextTurn->SetIcon(d_pic_nextTurn, 0, 0);
	b_move->SetIcon(d_pic_move, 0, 0);
	b_next->SetIcon(d_pic_next, 0, 0);
	b_defend->SetIcon(d_pic_defend, 0, 0);
	b_defendAndNext->SetIcon(d_pic_defendAndNext, 0, 0);
	b_search->SetIcon(d_pic_search, 0, 0);
	b_centerCurrent->SetIcon(d_pic_centerCurrent, 0, 0);

	//some other callbacks
    Playerlist::getInstance()->snextTurn.connect(SigC::slot((*this), &W_Edit::nextTurn));
    Playerlist::getInstance()->snextRound.connect(SigC::slot((*d_gameScenario), &GameScenario::nextTurn));
			
    d_allDefending = false;
    checkButtons();
}

W_Edit::~W_Edit()
{
    //smallmap as well as bigmap have timers running, so first stop the timers
    //and then wait some microseconds to make sure we don't get problems with
    //the timers
    stopTimers();
    usleep(100);

	delete d_smallmap;
	delete d_bigmap;

    delete d_gameScenario;
    
	delete d_stackinfo;
	delete l_turns;
	delete l_gold;
	SDL_FreeSurface(d_pic_next);
	SDL_FreeSurface(d_pic_defend);
	SDL_FreeSurface(d_pic_move);
	SDL_FreeSurface(d_pic_defendAndNext);
	SDL_FreeSurface(d_pic_search);
    SDL_FreeSurface(d_pic_nextTurn);
    SDL_FreeSurface(d_pic_centerCurrent);
    delete b_next;
	delete b_move;
	delete b_defend;
	delete b_defendAndNext;
	delete b_search;
	delete b_nextTurn;
	delete b_centerCurrent;
}

PARAGUI_CALLBACK(W_Edit::b_nextTurnClicked)
{
	Playerlist::getInstance()->getActiveplayer()->nextTurn();
	return true;
}

void W_Edit::nextTurn()
{
	debug("nextTurn()");

	if (!Playerlist::getActiveplayer())
    {
		return;
    }

	d_bigmap->stackDeselected();
	d_stackinfo->Hide();
	d_allDefending = false;
	displayFirstCity();
	updateStatus();
	
	// human player
	if (Playerlist::getActiveplayer()->getType() == Player::HUMAN)
    {
	        if (Configuration::s_showNextPlayer)
            {
                pictureNextPlayer();
            }
            //it is a bit ugly to place this code in three locations, but...
            else if (rand() % 15 == 0)
            {
                heroJoins(false);
                d_bigmap->Redraw();
            }
            return;
    }

    //quick hack: offer every non-human player a hero on occasion, replace it
    //with something more generic ASAP
    if ((Playerlist::getActiveplayer() != Playerlist::getInstance()->getNeutral())
        && (rand() % 15 == 0))
    {
        heroJoins(false);
        d_bigmap->Redraw();
    }
}

// Print new status
void W_Edit::updateStatus()
{
	char buffer[80];
	sprintf(buffer, "Turns: %i", d_gameScenario->getTurns());
	l_turns->SetText(buffer);

	sprintf(buffer, "Gold: %i", Playerlist::getActiveplayer()->getGold());
	l_gold->SetText(buffer);
}

// find first city of activeplayer and view it if outside screen
void W_Edit::displayFirstCity()
{
	debug("displayFirstCity()");

	for (Citylist::iterator it = Citylist::getInstance()->begin();
        it != Citylist::getInstance()->end(); it++)
	{
		if ((*it).getPlayer() == Playerlist::getInstance()->getActiveplayer())
		{
			d_bigmap->centerView((*it).getPos());
			break;
		}
	}
}

// pop up message window for next player
void W_Edit::pictureNextPlayer()
{
	debug("pictureNextPlayer()");

    d_nextPlayerPopup->setName(Playerlist::getInstance()->getActiveplayer()->getName());
    d_nextPlayerPopup->Show();
    SetCapture();
}

void W_Edit::heroJoins(bool free)
{
	debug("heroJoins()");

	// Calculate how much the hero will cost (partly random based)
	int needed_gold = 0;
	if (!free)
    {
        needed_gold = 800 + rand() % 700;
    }

	// check if the player has enough money enough gold
	if (Playerlist::getInstance()->getActiveplayer()->getGold() >= needed_gold)
	{
	    if (Playerlist::getInstance()->getActiveplayer()->getType() == Player::HUMAN)
	    {
		    Hero_offer hero_offer(needed_gold, this);
		    hero_offer.Show();
		    hero_offer.RunModal();
		    hero_offer.Hide();
		}
		else
		{
		    City* city = 0;
			// first city of activeplayer
        	for (Citylist::iterator it = Citylist::getInstance()->begin();
                it != Citylist::getInstance()->end(); it++)
        	{
        		if ((*it).getPlayer() == Playerlist::getInstance()->getActiveplayer())
        		{
        			city = &(*it);
        			break;
        		}
        	}
            assert(city);
			city->addHero(0, "ComputerLord");

			city->addHero(0, "ComputerLord2");
			city->addHero(0, "ComputerLord3");

			Playerlist::getInstance()->getActiveplayer()->withdrawGold(needed_gold);
		}
		updateStatus();
	}
}

PARAGUI_CALLBACK(W_Edit::b_nextClicked)
{
	Stack* stack = Playerlist::getActiveplayer()->getStacklist()->setNext();
    std::cerr << "Setting active stack = " << stack << "\n";
	if (stack) d_bigmap->stackSelected();
	else d_allDefending = true;
	checkButtons();
	return true;
}

PARAGUI_CALLBACK(W_Edit::b_moveClicked)
{
	Playerlist::getActiveplayer()->stackMove(Playerlist::getActiveplayer()->getActivestack());
	checkButtons();
	return true;
}

PARAGUI_CALLBACK(W_Edit::b_defendClicked)
{
	Playerlist::getActiveplayer()->getActivestack()->setDefending(true);
	d_bigmap->stackDeselected();
	d_stackinfo->Hide();
	checkButtons();

	return true;
}

PARAGUI_CALLBACK(W_Edit::b_defendAndNextClicked)
{
	b_defendClicked(GetID(), this, 1, 0);
	b_nextClicked(GetID(), this, 1, 0);
	return true;
}

PARAGUI_CALLBACK(W_Edit::b_centerCurrentClicked)
{
    Stack *stack = Playerlist::getActiveplayer()->getActivestack();
    if (stack) 
        d_bigmap->stackSelected();
    return true;
}

PARAGUI_CALLBACK(W_Edit::b_searchClicked)
{
	Stack* stack = Playerlist::getActiveplayer()->getActivestack();

	Player* p = Playerlist::getInstance()->getActiveplayer();
	Ruin* ruin = Ruinlist::getInstance()->getObjectAt(stack->getPos());
	Temple* temple = Templelist::getInstance()->getObjectAt(stack->getPos());

    Hero *hero = (Hero*)(stack->getFirstHero());
	assert(hero);

    std::cerr << "search: ruin = " << ruin << ", temple = " << temple
              << "\n";

	if (ruin)
	{
		int cur_gold = p->getGold();

		if (!(p->stackSearchRuin(stack, ruin)))
		{
			if (stack->empty())
			{
				stackDied(stack);
				Stacklist::deleteStack(stack);
				return true;
			}
			stackRedraw(GetID(), this, 1, 0);
			return true;
		}

		stackRedraw(GetID(), this, 1, 0);

		//this also includes the gold-only hack
		int gold_added = p->getGold() - cur_gold;

		RuinSearchDialog dialog(this, gold_added);
		dialog.Show();
		dialog.RunModal();
		dialog.Hide();

		updateStatus();
	}
	
	else if ((temple) && (temple->search()))
	{

        const unsigned int w1 = 200;
        const unsigned int h1 = 110;
		PG_MessageBox mb(this, PG_Rect((my_width-w1)/2, 
                                       (my_height-h1)/2, w1, h1),
                "Temple", "Select an action",
				PG_Rect(10, h1-40, 80, 30), "Bless",
				PG_Rect(100, h1-40, 80, 30), "Quest",
                PG_TA_CENTER
                );
		mb.Show();
		int decision = mb.WaitForClick();
        debug("decision = " << decision);
		mb.Hide();
        if (decision == 1)
        {
            unsigned int w1 = 240;
            unsigned int h1 = 110;
		    PG_MessageBox mb(this, PG_Rect((my_width-w1)/2, 
                                           (my_height-h1)/2, w1, h1),
                    "Temple", "Your armies have been blessed",
			    	PG_Rect(80, 70, 80, 30), "OK");
		    mb.Show();
		    mb.WaitForClick();
		    mb.Hide();
		    p->stackVisitTemple(stack, temple);
        }
        else
        {
            Quest *q = QuestsManager::getInstance()->createNewQuest(hero->getId());
            const unsigned int height = 160;
            const unsigned int width =  400;
            std::string title = "Quest - " + hero->getName();
            const unsigned int btn_w = 80;
            const unsigned int btn_h = 30;
            const char *txt = q ? q->getDescription()
                                : "This hero already has a quest";
		    PG_MessageBox mb(this, PG_Rect((my_width-width)/2, 
                                           (my_height-height)/2, 
                                           width, height),
                    title.c_str(), txt,
			    	PG_Rect((width - btn_w)/2, height - btn_h - 10, 
                            btn_w, btn_h), "OK");
   		    mb.Show();
		    mb.WaitForClick();
		    mb.Hide();
        }
	}

	return true;
}

// the parameter is currently not used (=0), but may be used for more detailed
// descriptions later on
void W_Edit::stackUpdate(Stack* s)
{
    std::cerr << "W_Edit stackUpdate: " << s << "\n";

    debug("W_Edit: stackUpdate()");

    std::cerr << "bigmap -> stackMoved: " << s << "\n";
	d_bigmap->stackMoved();
    std::cerr << "small map -> stackMoved: " << s << "\n";
	d_smallmap->stackMoved();

    std::cerr << "stackinfo -> readData: " << s << "\n";
	d_stackinfo->readData();

    std::cerr << "checkButtons: " << s << "\n";
	checkButtons();

    std::cerr << "finish stackUpdate " << s << "\n";

}

// s is currently unused, but can later be filled with reasonable data
void W_Edit::stackDied(Stack* s)
{
    debug("stackDied()");
	d_bigmap->stackDeselected();
	d_smallmap->Redraw();
	d_stackinfo->Hide();
	checkButtons();
}

PARAGUI_CALLBACK(W_Edit::stackRedraw)
{
	d_bigmap->Redraw();
	d_stackinfo->readData();
	checkButtons();
	return true;
}

void W_Edit::bigmapStackSelected(Stack* s)
{
	d_stackinfo->Show();
	d_stackinfo->readData();
	checkButtons();
}

void W_Edit::cityOccupied(City* city)
{
	debug("cityOccupied()");

	// if a computer makes it's turn and occupied a city, we shouldn't
    // show a modal dialog :)

	if (!d_lock)
	{
	    CityOccupationDialog dialog(city);
	    dialog.Show();
	    dialog.RunModal();
	    dialog.Hide();
    }
   
	Playerlist::getInstance()->checkPlayers();
	d_bigmap->Redraw();
	d_stackinfo->readData();
	checkButtons();
}

void W_Edit::bigmapRedraw()
{
	d_bigmap->Redraw();
}

void W_Edit::smallmapRedraw()
{
	d_smallmap->Redraw();
}

void W_Edit::endingTurn(Player::TYPE type)
{
    if (type != Player::HUMAN)
        return;
    lockScreen();
}

void W_Edit::lockScreen()
{
	debug("lockScreen()");

	//don't accept modifying user input from now on
	d_bigmap->setEnable(false);
	b_nextTurn->EnableReceiver(false);
	b_next->EnableReceiver(false);
	b_defend->EnableReceiver(false);
	b_move->EnableReceiver(false);
	b_search->EnableReceiver(false);
	b_defendAndNext->EnableReceiver(false);
	b_centerCurrent->EnableReceiver(false);
	d_lock = true;
    stopTimers();   //on computer turns, strange things may happen if the timers
                    //are still active
}

//the inverse of lock_screen()

void W_Edit::unlockScreen()
{
	debug("unlockScreen()");
	b_nextTurn->EnableReceiver(true);
	d_bigmap->setEnable(true);
	d_lock = false;
	checkButtons();
    startTimers();
}

void W_Edit::stopTimers()
{
    d_bigmap->interruptTimer();
    d_smallmap->interruptTimer();
}

void W_Edit::startTimers()
{
    //never start timers with the computer player's turn
    if (d_lock)
        return;

    d_bigmap->restartTimer();
    d_smallmap->restartTimer();
}

void W_Edit::checkButtons()
{
    if (d_lock) return;
    
	debug("checkButtons()");

	// first assume that all buttons are disabled
    bool move = false;
    bool defend = false;
    bool defendAndNext = false;
    bool search = false;
    bool next = false;
    bool center = false;

	// now check which buttons should be enabled
	if (!d_allDefending) 
	{
        next = true;
	}

	Stack* stack = Playerlist::getActiveplayer()->getActivestack();
	if (stack)
	{
        defend = true;
        center = true;

		if (stack->getPath()->size() > 0 && stack->getPlayer()->enoughMoves(stack))
		{
            move = true;
		}
		//TBD what if there is only 1 army and it isn't defending?
		// should be something like (!all defend) && (further_stacks)
        // Answer: Then the army defends and no stack is selected. This
        // behaviour sounds OK to me...
		if (!d_allDefending)
		{
			defendAndNext = true;
		}

		if (stack->hasHero())
		{
			Ruin* ruin = Ruinlist::getInstance()->getObjectAt(stack->getPos());
			Temple* temple = Templelist::getInstance()->getObjectAt(stack->getPos());
			if ((ruin && !ruin->isSearched()) || temple)
			{
                search = true;
			}
		}
	}

    // now set the button states according to the calculations
	b_move->EnableReceiver(move);
	b_defend->EnableReceiver(defend);
	b_defendAndNext->EnableReceiver(defendAndNext);
	b_search->EnableReceiver(search);
	b_next->EnableReceiver(next);
	b_centerCurrent->EnableReceiver(center);

    int x;
    next ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_next, SDL_SRCALPHA, x);
    move ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_move, SDL_SRCALPHA, x);
    defend ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_defend, SDL_SRCALPHA, x);
    defendAndNext ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_defendAndNext, SDL_SRCALPHA, x);
    search ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_search, SDL_SRCALPHA, x);
    center ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_centerCurrent, SDL_SRCALPHA, x);

	// redraw the buttons (SetIcon doesn't do it)
    // don't redraw everything, or bigmap is drawn again
    b_move->Redraw();
    b_move->Update();
    b_defend->Redraw();
    b_defendAndNext->Redraw();
    b_search->Redraw();
    b_next->Redraw();
    b_centerCurrent->Redraw();
}

void W_Edit::startGame()
{
	debug ("start_game()");
    lockScreen();
	Playerlist::getInstance()->nextPlayer();
}

void W_Edit::loadGame()
{
    if (Playerlist::getActiveplayer()->getType() == Player::HUMAN)
    {
        //human players want access to the controls and an info box
        unlockScreen();
        d_bigmap->stackDeselected();
    	d_stackinfo->Hide();
    	displayFirstCity();
	    updateStatus();

        string s = "Next Player: ";
        s += Playerlist::getInstance()->getActiveplayer()->getName();
    	PG_MessageBox mb(this, PG_Rect(x, y, 200, 110), "Next Player",
                s.c_str(), PG_Rect(60, 70, 80, 30), "OK");
    }       
    else
    {
        lockScreen();
        updateStatus();
        Playerlist::getActiveplayer()->startTurn();
    }
}

bool W_Edit::eventMouseButtonDown(const SDL_MouseButtonEvent* event)
{
    // only used to hide the "pictureNextPlayer" Box
	if(event->button == SDL_BUTTON_LEFT)
	{
        debug("eventMouseButtonDown: left button pressed");
        if (d_nextPlayerPopup->IsVisible())
        {
            d_nextPlayerPopup->Hide();
            ReleaseCapture();
 
            //quick hack: offer a hero on occasion, this is called to avoid
            //hassle when showing the next player AND getting a hero (capture)
            if (rand() % 15 == 0)
            {
                heroJoins(false);
                d_bigmap->Redraw();
            }
        }
	}
	return true;
}

// End of file
